/*  FREE_ALL.C
 *  this demonstrates how to free a clutch
 *  of allocated memory blocks in one operation
 *  last mod.: 11-FEB-90
 */

#include <STDIO.H>
#include <STDLIB.H>
#include <STRING.H>
#include <FCNTL.H>
#include <CONIO.H>
#include <IO.H>

#include "L_MEM.H"

#define MAX_SIZE 153600L

int err_flag;
char msg[50];

void main(int argc, char **argv);
Uint exhaust_memory(void);
void print_error(void);
void print_reports(void);

void main(int argc,char **argv)
{
Far_str_ptr ptr1, ptr2, ptr3;

srand(atoi(argv[1]));

printf("\nUse:  FREE_ALL <random_seed>"
"\nThis program demonstrates the use of mem_free_all() to free a clutch of"
"\nmemory blocks.  First some memory blocks are allocated.  Then memory is "
"\nallocated in blocks of random size (up to %dK) until an \"insufficient"
"\nmemory\" error is encountered.  The second set of blocks are then all"
"\nfreed at once using mem_free_all().  The previous blocks are resized and"
"\nare then freed, again using mem_free_all().\n",
MAX_SIZE/1024);

printf("\nLargest block available = %lu",mem_max(&err_flag));

/*  turn the memory allocation log on  */
mem_log_on();
/*  terminate if control blocks destroyed  */
crit_err_termination_on();
/*  on critical error send report #1 to the printer  */
set_crit_err_termination_output(PRINTER_OUTPUT,REPORT_1_ONLY);

printf("\n3 memory blocks allocated.");
ptr1 = mem_alloc(12000L,&err_flag);
if ( !err_flag )       /*  if no error  */
    {
    ptr2 = mem_alloc(15000L,&err_flag);
    if ( !err_flag )
        ptr3 = mem_alloc_ch(5000L,0xFF,&err_flag);
        /*  omit "ptr3 = " and thus change this to
         *  mem_alloc_ch(5000L,0xFF,&err_flag);
         *  to create a situation in which a critical error will
         *  (probably occur) causing report #1 to be sent to the printer
         */
    }
if ( err_flag )
    {
    printf("\nProblem allocating memory.");
    print_error();
    exit(1);
    }

/*  call a function in which memory blocks are allocated
 *  and later freed by a call to mem_free_all()
 */
printf("\n%d more memory blocks allocated ... ",exhaust_memory());
printf("and freed.");

/*  resize previously allocated memory blocks  */
ptr1 = mem_increase(ptr1,20000,&err_flag);
ptr2 = mem_decrease(ptr2,10000,&err_flag);
ptr3 = mem_resize_ch(ptr3,10000L,0xAA,&err_flag);
printf("\nFirst 3 blocks resized ... ");

/*  now free up all allocated memory  */
mem_free_all(0,&err_flag);
if ( !err_flag )
    printf("and freed.");

printf("\n\nThis is report #3 (summary from report #2):");
mem_log_report_3(stdout,0,FALSE,&err_flag);
print_reports();

printf("\nLargest block available = %lu\nNow erasing log ...",
    mem_max(&err_flag));

/*  now throw away the memory log  */
mem_erase_log(&err_flag);

if ( err_flag )
    {
    printf("\nWhen erasing memory log: ");
    print_error();
    exit(2);
    }

printf("\nLargest block available = %lu\n",mem_max(&err_flag));
}

/*  perform a bunch of memory allocations until memory is exhausted  */
Uint exhaust_memory(void)
{
Uint n = 0;
int block_id = get_current_block_id();
/*  save ID of most recently allocated block
 *  so mem_free_all() knows where to free from
 */

mem_log_msg("Now allocating blocks until no more space ...",&err_flag);
err_flag = NO_ERROR;
while ( err_flag != INSUFFICIENT_MEMORY )
    {
    mem_alloc_ch((((Ulong)rand())*rand())%MAX_SIZE,(char)rand(),&err_flag);
    if ( err_flag == NO_ERROR )
        n++;
    if ( err_flag != INSUFFICIENT_MEMORY && err_flag != NO_ERROR )
        {
        print_error();
        mem_log_report_2(stdout,1,0,&err_flag);
        exit(3);
        }
    }

/*  now free all blocks allocated using exhaust_memory()
 *  with a call to mem_free_all()
 */
mem_free_all(block_id,&err_flag);
if ( err_flag )
    {
    printf("\nWhen freeing allocated blocks: ");
    print_error();
    exit(4);
    }

return ( n );   /*  number of blocks allocated (and freed)  */
}

void print_error(void)
{
if ( err_flag )
    printf("\n%s",mem_err_msg(err_flag,msg));
}

void print_reports(void)
{
int ch;
FILE *output;
char *logfile = "FREE_ALL.LOG";

printf("\nPress P to send reports #1 and #2 to printer,"
       "\nF to send reports to disk file %s,"
       "\nany other key to quit ... ",logfile);

while ( kbhit() )
    getch();
ch = getche();
if ( ch == 'P' || ch == 'p' )
    {
    setmode(fileno(stdprn),O_TEXT);
    mem_log_report_1(stdprn,1,TRUE,&err_flag);
    mem_log_report_2(stdprn,1,TRUE,&err_flag);
    }
else if ( ch == 'F' || ch == 'f' )
    {
    if ( ( output = fopen(logfile,"wt") ) == NULL )
        {
        printf("\nUnable to open file %s",logfile);
        exit(5);
        }
    mem_log_report_1(output,1,TRUE,&err_flag);
    if ( err_flag )
        printf("\nError when writing first report: %s",
            mem_err_msg(err_flag,msg));
    else
        {
        mem_log_report_2(output,1,TRUE,&err_flag);
        if ( err_flag )
            printf("\nError when writing second report: %s",
            mem_err_msg(err_flag,msg));
        }
    }

printf("\n");
}
